@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::SpamAssassin;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-use version; our $VERSION = version->declare('v1.1.1');
+use version; our $VERSION = version->declare('v1.1.3');
 
 use English qw{ -no_match_vars };
 use Sys::Syslog qw{:standard :macros};
@@ -29,6 +29,13 @@ sub default_config {
     }
 }
 
+sub grafana_rows {
+    my ( $self ) = @_;
+    my @rows;
+    push @rows, $self->get_json( 'SpamAssassin_metrics' );
+    return \@rows;
+}
+
 sub register_metrics {
     return {
         'spamassassin_total' => 'The number of emails processed for SpamAssassin',
@@ -276,7 +283,7 @@ Marc Bradshaw E<lt>marc@marcbradshaw.netE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2015
+Copyright 2017
 
 This library is free software; you may redistribute it and/or
 modify it under the same terms as Perl itself.
